--* Begin List of Functions & Procedures *
---> File: C:\EUPHORIA\INCLUDE\QTEXT.E
--global function Pad(sequence s,integer len,integer mode)
--global function IsBlankLine(sequence s)
--global function Strip(sequence s,sequence delimeters,integer mode)
--global function StripComment(sequence s)
--global function IsAllSame(sequence s)
--global procedure PlaceLineAt(sequence RC,object o)
--global procedure PlaceAt(sequence RC,object o)
--global procedure CenterAt(integer line,sequence s)
--global procedure BlankLine(integer line)
--global procedure Pause(sequence msg,integer mode)
--global procedure Msg(sequence msg)
--global function Response(sequence msg,sequence test)
--global function Input( integer size )
--global procedure Box(sequence RC)
--global procedure QBox(sequence RC,sequence s)
--global function SplitLine(sequence s,integer maxWidth)
--global function FindToken(sequence toFind,sequence s,sequence delimeters)
--global function Tokenize(sequence s,sequence delimeters)
--global function Delete(integer mark,sequence s)
--global function Insert(sequence s,object new,integer here)
--global function CountWords(sequence s)
--global function Lower(object o)
--global function Upper(object o)
--global function GetFile(sequence fName)
--global function WriteFile(sequence fName,sequence s)
--* End List of Functions & Procedures   *
--qtext.e
--By Quark  8/99 and after - A set of Quark-written or gathered text routines
include get.e
include graphics.e
include machine.e --for tick_rate() in Pause()
include qconsts.e
include qtypes.e
------------------------------
--establishment of MAXROW & MAXCOL for screen
sequence vc
vc=video_config()
global constant MAXROW=vc[VC_LINES],MAXCOL=vc[VC_COLUMNS]
----------------------------------------
global function Pad(sequence s,integer len,integer mode)
--Quark: Add spaces to s to length len in indicated mode (left,right,center).
if mode=LEFT then
   while length(s)<len do
      s=' '&s
   end while
elsif mode=RIGHT then
   while length(s)<len do
      s&=' '
   end while
else --must be center
   while length(s)<len do
      s=s&' '
      if length(s)<len then
         s=' '&s
      end if
   end while
end if
return s
end function
----------------------------------------
global function IsBlankLine(sequence s)
  for i=1 to length(s) do
    if not isWhitespace(s[i]) then
       return FALSE
    end if
  end for
return TRUE
end function
------------------------------
global function Strip(sequence s,sequence delimeters,integer mode)
--Quark: Strip whitespace from s in mode way (0=left,1=right,2=both)
if equal(mode,0) or equal(mode,2)then
   while length(s) and find(s[1],delimeters) do
      if equal(length(s),1) then
         s={}
      else
         s=s[2..length(s)]
      end if
   end while
end if
if equal(mode,1) or equal(mode,2) then
   while length(s) and find(s[length(s)],delimeters) do
      if equal(length(s),1) then
         s={}
      else
         s=s[1..length(s)-1]
      end if
   end while
end if
return s
end function
---------------------------------
global function StripComment(sequence s)
--Returns line stripped of comment and all standard DELIMETERS
integer f
f=match("--",s)
if f then
   if f>1 then
      return Strip(s[1..f-1],DELIMETERS,2)
   else
      return {}
   end if
else
   return Strip(s,DELIMETERS,2)
end if
end function
------------------------------
global function IsAllSame(sequence s)
--Quark: Returns TRUE if all characters the same, FALSE otherwise
--Ignores '\n'
   atom a
   if atom(s) or length(s)=1 then
      return TRUE
   end if
   a=s[1]
   for i=1 to length(s) do
      if not equal(a,s[i]) and not equal(s[i],'\n') then
         return FALSE
      end if
   end for
   return TRUE
end function
----------------------------------------
global procedure PlaceLineAt(sequence RC,object o)
--Quark: Screen-print a line of chars.
-- RC={r1,c1,r2,c2}; o=chars sequence or atom; hv=orientation (horiz, vert)
-- If length(o)<4, then assume box-line & generate it, else string.
integer len,lenO,hv
   if RC[1]=RC[3] then -- Is it horizontal?
      hv=HORIZONTAL
   else
      hv=VERTICAL -- No, vertical.
   end if
   if atom(o) then
      o={o}
   end if
   if hv=HORIZONTAL then
      len=RC[4]-RC[2]+1
   else
      len=RC[3]-RC[1]+1
   end if
   lenO=length(o)
   if lenO=0 then
      o=repeat(' ',len)
   elsif lenO=1 then
      o=repeat(o[1],len)
   elsif lenO=2 then
      o=o[1]&repeat(o[2],len-1)
   elsif lenO=3 then
      o=o[1]&repeat(o[2],len-2)&o[3]
   else
      if lenO>len then
         o=o[1..len]
      end if
      if lenO<len then
         o=Pad(o,len,CENTER)
      end if
   end if
   if hv=HORIZONTAL then --horizontal
      position(RC[1],RC[2])
      puts(1,o)
   else
      for i=1 to length(o) do
         position(RC[1]+i-1,RC[2])
         puts(1,o[i])
      end for
   end if
end procedure
----------------------------------------
global procedure PlaceAt(sequence RC,object o)
--Quark: multi-purpose screen-print function: if RC is length 2, then print
-- normally; else, use the elaborate PlaceLineAt() function.
   if length(RC)=2 then
      position(RC[1],RC[2])
      puts(1,o)
   else
      PlaceLineAt(RC,o)
   end if
end procedure
----------------------------------------
global procedure CenterAt(integer line,sequence s)
--Quark: Center a line using padding.  Use line #0 if line is pre-positioned.
--Note: MAXCOL must be pre-defined.
sequence at
if equal(s,{}) then
   --do nothing
else
   if line=0 then
      at=get_position()
      line=at[1]
   end if
   PlaceAt({line,2,line,MAXCOL-1},s)
end if
end procedure
----------------------------------------
global procedure BlankLine(integer line)
   PlaceAt({line,2,line,MAXCOL-1}," ")
end procedure
----------------------------------------
global procedure Delay(integer m)
atom t
t=time()
while time()<t+m do
end while
end procedure
----------------------------------------
global procedure Pause(sequence msg,integer mode)
--Quark: multi-function Pause
--if mode=-1 then print nothing, just wait until a key is pressed
--if mode=0 then print msg on message line & wait for key-press
--else print msg & wait for mode hundredths of a second, erase message
atom t,m
if equal(mode,-1) then
   while not wait_key() do
   end while
elsif equal(mode,0) then
   CenterAt(MAXROW,msg)
   while not wait_key() do
   end while
else
   tick_rate(100)
   m=mode*(1/100)
   CenterAt(MAXROW,msg)
   t=time()
   while time()<t+m do
   end while
   BlankLine(MAXROW)
end if
end procedure
----------------------------------------
global procedure Msg(sequence msg)
--Quark: simple message on MAXROW to user
   CenterAt(MAXROW,msg)
end procedure
----------------------------------------
global function Response(sequence msg,sequence test)
--Quark: One-key response w msg -- key must be in sequence test
   integer k,x
   Msg(msg)
   x=0
   while not x do
      k=wait_key()
      x=find(k,test)
   end while
   BlankLine(MAXROW)
   return k
end function
----------------------------------------
global function Input( integer size )
--Author: David Cuny
-- get text from user at current position
-- maximum length is size
 integer row, col, key
 sequence at, text
 -- get the current position
 at = get_position()
 row = at[1]
 col = at[2]
 -- set text
 text = ""
 -- main loop
 while 1 do
     -- show the text
     position( row, col )
     -- border
     puts( 1, "["                                    -- left border
         & text                                      -- data
         & repeat( FILL_CHAR, size - length(text) )  -- filler
         & "]" )                                     -- right border
     -- position cursor
     position( row, col+length( text )+1 )
     -- get a key
     key = wait_key()
     -- process the key
     if key = ESC then
         text = ""
         exit
     elsif key = ENTER then
         exit
     elsif key = BACKSPACE then
         -- handle backspace
         if length( text ) > 0 then
             -- remove the last character from the text
             text = text[1..length(text)-1]
         end if
     elsif key >= ' ' and   key <= '~' then
         -- handle "normal" key
         if length( text ) < size then
             text = text & key
         end if
     else
         -- ignore key
      end if
 end while
 return text
end function
----------------------------------------
global procedure Box(sequence RC)
--Quark: plain box.  RC={r1,c1,r2,c2}
   object o
   sequence rc
   --Top line
   rc={RC[1],RC[2],RC[1],RC[4]} --span between cordinates
   o={PART[1][1],PART[1][2],PART[1][5]} -- top line
   PlaceAt(rc,o) -- Print line
   --Verical lines
   rc={RC[1]+1,RC[2],RC[3]-1,RC[2]} -- 1st vertical
   o=PART[2][1]
   PlaceAt(rc,o)
   rc={RC[1]+1,RC[4],RC[3]-1,RC[4]} -- 2nd vertical
   PlaceAt(rc,o)
   --Bottom line
   rc={RC[3],RC[2],RC[3],RC[4]} --bottom line
   o={PART[5][1],PART[1][2],PART[5][5]} -- bottom line
   PlaceAt(rc,o)
end procedure
----------------------------------------
global procedure QBox(sequence RC,sequence s)
--Quark: Draw box with header sub-box.  data: r1,c1,r2,c2 & header: s
sequence rc,o
Box(RC)
rc={RC[1]+2,RC[2],RC[1]+2,RC[4]} --drop to 3rd line
o={PART[3][1],PART[1][2],PART[3][5]} --horizontal line at row 3
PlaceAt(rc,o) -- Print sub-box line
rc={RC[1]+1,RC[2]+1,RC[1]+1,RC[4]-1}
PlaceAt(rc,s) -- header
end procedure
----------------------------------------
global function SplitLine(sequence s,integer maxWidth)
--Quark: Split a string into two parts at a whitespace if possible
   sequence t1,t2
   t1={}
   t2={}
   if length(s)<maxWidth then
      return {s,{}}
   end if
  for i=maxWidth to maxWidth/2 by -1 do
       if not atom(s[i]) then
               printf(1,"Error: %d %s",{i,s[i]&'\n'})
               ? s[i]
               abort(0)
       end if
       if isWhitespace(s[i]) then
               t1=Strip(s[1..i],DELIMETERS,1)&'\n'
               if atom(t1) then
                  t1={t1}
               end if
               if equal(t1,{}) then
                 t1={' '}
               end if
               t2=Strip(s[i+1..length(s)],DELIMETERS,0)
               if atom(t2) then
                  t2={}
               end if
               if equal(t2,{}) then
                 t2={' '}
               end if
               return {t1,t2}
       end if
  end for
  t1=Strip(s[1..maxWidth],DELIMETERS,1)&'\n'
  if atom(t1) then
   t1={t1}
  end if
  if equal(t1,{}) then
     t1={' '}
  end if
  t2=Strip(s[maxWidth+1..length(s)],DELIMETERS,0)
  if atom(t2) then
   t2={t2}
  end if
  if equal(t2,{}) then
     t2={' '}
  end if
  return {t1,t2}
end function
------------------------------
global function Insert(sequence s,object new,integer here)
--Quark: Insert an object in a sequence
   return append(s[1..here-1],new)&s[here..length(s)]
end function
----------------------------------------
global function MaxLengthLines(integer max,sequence s)
--Quark: Returns s such that no line is longer than max
integer count
sequence t
t={}
count=0
while 1 do
  count+=1
  if count>length(s) then exit  end if
  if not atom(s[count]) then
    if length(s[count])>max then
      t=SplitLine(s[count],max)
      s[count]=Strip(t[1],DELIMETERS,1)
      --Insert remainder of long line
      s=Insert(s,Strip(t[2],DELIMETERS,1),count+1)
    end if
  end if
end while
return s
end function
----------------------------------------
global function FindToken(sequence toFind,sequence s,sequence delimeters)
--Quark: Finds a token (series of chars) between delimeters
--Returns TRUE if found or FALSE if not found
--Example: if findToken("procedure",data[i],delimeters) then
    sequence token
    token={}
    for i=1 to length(s) do
        if find(s[i],delimeters) then
            if equal(token,toFind) then
                return TRUE
            end if
            token={}
        else
            token&=s[i]
        end if
    end for
    return FALSE
end function
----------------------------------------
global function Tokenize(sequence s,sequence delimeters)
--Quark: Return list of all tokens (chars between delimeters) in s
    sequence list,token
    token={}
    list={}
    for i=1 to length(s) do
       if not find(s[i],delimeters) then
          token&=s[i]
       else
         if length(token) then
            list=append(list,token)
            token={}
         end if
       end if
    end for
    return list
end function
----------------------------------------
global function Explode(object o)
   --Quark: Break any line into sequences of either punctuation or groups
   --of non-punctuation characters.  Whitespace is the delimeter.
   --e.g. StripComment(o) comes back {{StripComment},{(},{o},{)}}
   --For Explode() of Euphoria lines, do StripComment first.
   integer x
   sequence whole,part
   whole={}
   if atom(o) then
      return o
   else
      if equal(o,{}) then
         return o
      end if
      x=1
      while x<=length(o) do
         while isWhitespace(o[x]) do
            x+=1
         end while
         if x<=length(o) then
            if isPunctuation(o[x]) then
               whole=append(whole,{o[x]})
               x+=1
            else
               part={}
               while x<=length(o) and
                (not isWhitespace(o[x]) and not isPunctuation(o[x])) do
                  part&=o[x]
                  x+=1
               end while
               whole=append(whole,part)
            end if
         end if
      end while
   end if
   return whole
end function
----------------------------------------
global function Delete(integer mark,sequence s)
--Quark: Cut an object from a sequence
   if mark<1 or mark>length(s) then
      return s
   elsif length(s)=1 then
      return {}
   else
      if equal(mark,1) then
         return s[2..length(s)]
      elsif equal(mark,length(s)) then
         return s[1..length(s)-1]
      else
         return s[1..mark-1]&s[mark+1..length(s)]
      end if
   end if
end function
------------------------------
global function Clip(sequence s, integer mark,integer len)
--Quark: Clip a sequence of chars from a string from mark through len
if mark>length(s) then
   return s
end if
if mark=1 then
   if len>=length(s) then
      return {}
   else
      return s[len+1..length(s)]
   end if
elsif (mark+len-1)>=length(s) then
   return s[1..mark-1]
else
   return s[1..mark-1]&s[mark+len..length(s)]
end if
end function
------------------------------
global function CountWords(sequence s)
--Quark: Counts words by counting whitespace groups
--Faster than using Tokenize?
    integer count,toggle
    count=0
    toggle=0 --Turn on once per group of whitespaces
    s=Strip(s,DELIMETERS,0)
    for i = 1 to length(s) do
        if isWhitespace(s[i]) then
            if not toggle then
                count+=1
                toggle=1
            end if
        else
            toggle=0
        end if
    end for
    return count
end function
------------------------------
global function Lower(object o)
--Quark: another lower() for char or string.
    integer diff
    diff='a'-'A'
    if atom(o) then
      o+= diff*(o>='A' and o<= 'Z')
      return o
    else
       for i = 1 to length(o) do
         o[i]+= diff*(o[i]>='A' and o[i]<= 'Z')
       end for
       return o
    end if
end function
------------------------------
global function Upper(object o)
--Quark: another upper() for char or string.
    integer diff
    diff='a'-'A'
    if atom(o) then
      o-= diff*(o>='z' and o<= 'z')
      return o
    else
       for i = 1 to length(o) do
         o[i]-= diff*(o[i]>='a' and o[i]<= 'z')
       end for
       return o
    end if
end function
----------------------------------------
global function GetFile(sequence fName)
--Quark: Read file into sequence of strings
integer fn
sequence data
object line
data={}
fn=open(fName,"r")
while 1 do
    if fn=EOF then
        Pause("Read from "&fName&" failed--press key",0)
        --abort(1)
        return {PROBLEM,data}
    else
        line = gets(fn)
        if atom(line) then
            close(fn)
            exit
        else
            data=append(data,line)
        end if
    end if
end while
return {NOPROBLEM,data}
end function
----------------------------------------
global function WriteFile(sequence fName,sequence s)
--Quark: Write all strings to a file
integer fn
fn=open(fName,"w")
if (fn=EOF) then
   Pause("Write to "&fName&" failed--Press key",0)
   return PROBLEM
else
   for i=1 to length(s) do
      puts(fn,s[i])
   end for
end if
close(fn)
return NOPROBLEM
end function
------------------------------

