--======================================================================
--Scroll Text V1.0 by Quark  2/2000  Written in Euphoria V2.2
-- Purpose: Presents text files to the user in a smooth-scrolling manner.
-- Maintains bookmarks so user can continue reading easily.
-- Old bookmarks can and should be deleted.
-- Requires text files with a ".txt" extension.
-- Thanks to Jiri Babor for his putsxy.e used for graphics char printing.
--======================================================================
--with profile_time
--with trace
without type_check
include graphics.e
include image.e
include putsxy.e --by Jiri Babor
include qtext.e  --by Quark
include qfs.e    --by Quark

constant U = 1048576
integer x,line,state,scrolledLines
integer xA1,xA2,yA1,yA2,xB1,xB2,yB1,yB2
sequence s,path,fname,bookmarks,f,currentDir
-------------------------------------
procedure ClearPrintArea()
--Quark: specialized function to clear scroll area
polygon(BLACK,1,{{xB1,yB1},{xB2,yB1},{xB2,yB2},{xB1,yB2}})
end procedure
-------------------------------------
function CalcLine(integer change,sequence s,integer curLine)
--Quark: specialized function to verify line count
integer x
ClearPrintArea()
x=scrolledLines
if equal(compare(x,length(s)),1) then
  return 0
end if
if equal(compare((curLine + change),2),-1) then
  return 0
elsif equal(compare((curLine + change),length(s) - (scrolledLines -1)),1) then
  return length(s)-scrolledLines-1
else
  return curLine + change-1  
end if
end function
-------------------------------------
global function ScrollText(sequence s,integer at)
--Quark: scrolls text lines in s. Uses graphics mode 18
--Uses putsxy() by Jiri Babor
sequence t,v,image,msg
integer maxRow,maxCol
integer fontHeight,fontWidth,count,key,printAt,gScreenLines
integer parts,toggle,unit

unit=floor(U/16)
toggle=0
--Clean lines - no whitespace on right and no line longer than 78 chars
Msg("Loading & re-arranging text file...takes a while on long files.")
for i=1 to length(s) do
  t=Strip(s[i],DELIMETERS,1)
  if not length(t) then
    t={' '}
  end if
  s[i]=t
end for
s=MaxLengthLines(78,s)
v={}
image={}
t={}

--Set graphics mode
if graphics_mode(18) then -- try other modes
   puts(1, "Mode Failure\n")
   abort(100)
end if

v=video_config()
maxCol=v[VC_XPIXELS]-1
maxRow=v[VC_YPIXELS]-1
fontHeight=16
fontWidth=8
parts=8
xA1=0
xA2=maxCol
yA1=0
yA2=maxRow
gScreenLines=floor((maxRow+1)/fontHeight)
xB1=xA1+fontWidth
xB2=xA2-fontWidth
scrolledLines=floor(floor(maxRow*.8)/fontHeight) --num lines printed
yB1=floor(gScreenLines/parts)*fontHeight -- 1/parts down the screen
yB2=yB1+scrolledLines*fontHeight-1
printAt=yB2-fontHeight
bk_color(BLACK)
clear_screen()
polygon(YELLOW,0,{{xB1-4,4},{xB2+4,4},{xB2+4,fontHeight+fontHeight+2},{xB1-4,fontHeight+fontHeight+2}})
t="S C R O L L   T E X T"
putsxy({floor((xA2-fontWidth*length(t))/2),fontHeight-2},t,YELLOW,CLEAR,'a')
polygon(YELLOW,0,{{xB1-4,yB1-fontHeight},{xB2+4,yB1-fontHeight},{xB2+4,yB2+fontHeight},{xB1-4,yB2+fontHeight}})
draw_line(YELLOW,{{xB1,yA2-fontHeight-4},{xB2,yA2-fontHeight-4}})
msg="<SPC> pause    <1,2,...8,9> speed    <HM, END, PGUP, PGDN> move  <ESC> done"
putsxy({floor((xA2-fontWidth*length(msg))/2),yA2-fontHeight},msg,GREEN,CLEAR,'a')
count=at-1

while 1 do
  for i=0 to yB2 do
    if not remainder(i,fontHeight) then
      count+=1
      if equal(compare(count,length(s)),1) then
	t=" *  *  *  *  *  *  *  *  END OF TEXT *  *  *  *  *  *  *  *"
      else
	t=s[count]
      end if
      putsxy({xB1,printAt},t,GREEN,CLEAR,'a')
    end if
    image=save_image({xB1,yB1+1},{xB2,yB2})
    display_image({xB1,yB1},image)

    --Delay
    for k=1 to unit do 
      if equal(toggle,0) then
	toggle=1
      else
	toggle=0
      end if
    end for --delay

    key=get_key()
    if equal(key,-1) then
      --do nothing      
    elsif equal(key,27) then
      t=path
      t=fname
      line=CalcLine(-scrolledLines,s,count)
      line=line+equal(line,0)
      bookmarks&={{path,fname,{line}}}
      exit
    elsif equal(key,'1') then
      unit=U
    elsif equal(key,'2') then
      unit=floor(U/2)
    elsif equal(key,'3') then
      unit=floor(U/4)
    elsif equal(key,'4') then
      unit=floor(U/8)
    elsif equal(key,'5') then
      unit=floor(U/16)
    elsif equal(key,'6') then
      unit=floor(U/32)
    elsif equal(key,'7') then
      unit=floor(U/64)
    elsif equal(key,'8') then
      unit=floor(U/128)
    elsif equal(key,'9') then
      unit=1
    elsif equal(key,HOME) then
      count=CalcLine(-count,s,count)
    elsif equal(key,END) then
      count=CalcLine(length(s)-count,s,count)
    elsif equal(key,PGUP) then
      count=CalcLine(-scrolledLines,s,count)
    elsif equal(key,PGDOWN) then
      count=CalcLine(scrolledLines,s,count)
    elsif equal(key,32) then
      key=0
      putsxy({floor((xA2-fontWidth*78)/2),yA2-fontHeight},repeat(' ',78),BLACK,BLACK,'a')
      putsxy({floor((xA2-fontWidth*length("P A U S E D"))/2),yA2-fontHeight},"P A U S E D",GREEN,CLEAR,'a')
      while not equal(key,32) do
	key=get_key()
      end while
      putsxy({floor((xA2-fontWidth*78)/2),yA2-fontHeight},repeat(' ',78),BLACK,BLACK,'a')
      putsxy({floor((xA2-fontWidth*length(msg))/2),yA2-fontHeight},msg,GREEN,CLEAR,'a')
    end if
  end for
  
  if equal(key,27) then
    exit
  end if
  if equal(compare(count,length(s)+scrolledLines),1) then
    exit    
  end if
end while

if graphics_mode(-1) then end if
return NOPROBLEM

end function
----------------------------------------
function MainScreen()
--Quark: specialized function: prints main screen
sequence m1,rc
m1={
" SCROLL TEXT is a smooth-scrolling text reader -- less headaches!",
" A bookmark is saved automatically when you stop reading the text.",
" Here are the options for SCROLL TEXT:",
" ",
" ",
"  1) Load a text file to read. Note: the file must have a `.txt' ending.",
" ",
"  2) Load a bookmarked file to continue reading (latest at bottom of list).",
" ",
"  3) Delete old bookmarks. These build up, so clean them out periodically.",
" ",
"  0) Quit SCROLL TEXT."
}

clear_screen()
rc={1,1,MAXROW-2,MAXCOL}
QBox(rc,"Quark's SCROLL TEXT")
Pause("This is the message line...",125)

for i=1to length(m1) do
   PlaceAt({i+4,3},m1[i])
end for

return (Response(" Enter your choice <1-3 or 0>: ","1,2,3,0"))-'0'

end function
----------------------------------------
function ChooseBookmark()
--Quark: Specialized function to select from a bookmarks file
--A bookmarks file is a series of { {{path},{fname},{lineNum}},... }
--Returns {{path},{fname},{lineNum}}
sequence bookmark
object t
bookmark={}
if not length(bookmarks) then
  Pause("Choose Function message: There are no bookmarks.",175)
  return {}
else    
  --Choose a bookmark
  for i=1 to length(bookmarks) do
    bookmark=append(bookmark,bookmarks[i][2]&' '&sprintf("%d",bookmarks[i][3]))
  end for
  Msg("<ESC> cancel   <UP DOWN PGUP PGDOWN HM END> navigate   <ENTER> select")
  Box({17,19,23,61})
  bookmark=ListSelect({18,20,22,60},bookmark,0)
  BlankLine(MAXROW)
  bookmark=bookmark[1]
  if length(bookmark) then
  
    for i= 1 to length(bookmark) do
      if equal(bookmark[i],' ') then
	t=value(bookmark[i+1..length(bookmark)])
	line=t[2]
	bookmark=bookmark[1..i-1]
	exit
      end if
    end for
    for i=1 to length(bookmarks) do
      if equal(bookmark,bookmarks[i][2]) and equal(line,bookmarks[i][3]) then
  
	return bookmarks[i]
      end if
    end for
  end if
end if
  return {}
end function
----------------------------------------
procedure DeleteBookmark()
--Quark: Specialized function to delete from a bookmarks file
--A bookmarks file is a series of { {{path},{fname},{lineNum}},... }
--Returns remaining bookmarks file: {{path},{fname},{lineNum}}
integer count
sequence bookmark,bmark,temp,t
if not length(bookmarks) then
  Pause("Delete Function message: There are no bookmarks.",175)
else    
  bookmark={}
  --Choose a bookmark
  for i=1 to length(bookmarks) do
    bookmark=append(bookmark,bookmarks[i][2]&' '&sprintf("%d",bookmarks[i][3]))
  end for
  Msg("<ESC> cancel  <UP DN PGUP PGDN HM END> navigate  <SPC> select  <ENTER> done")
  Box({17,19,23,61})
  bookmark=ListSelect({18,20,22,60},bookmark,1)
  for i=1 to length(bookmark) do
    bmark=bookmark[i]
    if length(bmark) then
      for j= 1 to length(bmark) do
	if equal(bmark[j],' ') then
	  t=value(bmark[j+1..length(bmark)])
	  line=t[2]
	  temp=bmark[1..j-1]
	  count=0
	  while equal(compare(count,length(bookmarks)),-1) do
	    count+=1
	    if equal(temp,bookmarks[count][2]) and equal(line,bookmarks[count][3]) then

	      bookmarks=Delete(count,bookmarks)
	      exit
	    end if
	  end while
	end if
      end for
    end if
  end for
end if
end procedure
----------------------------------------
function SaveBookmarks()
--Quark: specialized function to save a bookmarks file
  sequence t
  t={}
  if not length(bookmarks) then
    Pause("Save Function message: There are no bookmarks.",175)
    x=open(currentDir&"\\bookmark.bmf","w")
    close(x)
    return PROBLEM  
  end if
  Pause("Saving bookmarks file...",175)
  x=open(currentDir&"\\bookmark.bmf","w")
  
  if not equal(x,-1) then
    for i=1 to length(bookmarks) do
      puts(x,bookmarks[i][1]&'\n')
      puts(x,bookmarks[i][2]&'\n')
      printf(x,"%d\n",bookmarks[i][3])
    end for
    close(x)
    return NOPROBLEM
  else
    Pause("Save Function message: Problem saving bookmarks file...",175)
    return PROBLEM
  end if
end function
----------------------------------------
function LoadBookmarks()
--Quark: specialized function to load bookmarks file
integer count,x,n
object t

Pause("Load Function message: Loading bookmarks file...",175)
x=open(currentDir&"\\bookmark.bmf","r")

if not equal(x,-1) then
  bookmarks={}
  count=0
  while 1 do
    count+=1
    for i=1 to 3 do
      t=gets(x)
      if atom(t) then
	if equal(count,1) then
	  Pause("Load Function message: ",100)
	  Pause("Bookmark file missing or empty -- bookmarks saved after reading a file.",200)
	end if
	exit 
      end if
      t=Strip(t,DELIMETERS,1)
      if equal(i,3) then
	t=value(t)
	n=t[2]
	bookmarks[count]=append(bookmarks[count],n)
      else
	if equal(i,1) then
	  bookmarks=append(bookmarks,{})
	end if
	bookmarks[count]=append(bookmarks[count],t)
      end if
    end for
    if atom(t) then exit end if
  end while
  close(x)
  return NOPROBLEM
else
  Pause("Load Function message: Problem with bookmark file.",200)
  return PROBLEM
end if
end function
----------------------------------------
--===================================================================
--Begin Program
bk_color(GREEN)
text_color(BLACK)
currentDir={}
currentDir=current_dir()
bookmarks={}
state=1
while state do

  bk_color(GREEN)
  text_color(BLACK)
   state = MainScreen() --Get user choice

   if state=0 then
      exit
   elsif state=1 then
      Pause("Select a file to read.",200)
      f={}
      f=fs() --major routine allowing file selection
      if equal(f[1],"HasFiles") then
	path=Lower(f[2])
	fname=Lower(f[3][1])
	--line=1
	if match(".txt",fname) then
	  s=GetFile(path&fname)
	  if equal(s[1],NOPROBLEM) then
	    x=ScrollText(s[2],1)  
	    x=SaveBookmarks()
	  else
	    Pause("Could not read file.",175)          
	  end if
	else
	    Pause("Problem with file -- must have `.txt' extention",175)          
	end if
      end if
    elsif state=2 then
      x=LoadBookmarks()
      if not  equal(x,PROBLEM) then
	s=ChooseBookmark()
	if length(s) then
	  path=s[1]
	  fname=s[2]
	  line=s[3]
	  s=GetFile(path&fname)
	  if equal(s[1],NOPROBLEM) then
	     s=s[2]
	     x=ScrollText(s,line)
	     x=SaveBookmarks()
	  else
	    Pause("No file chosen.",175)
	  end if
	end if
      end if
    else
      x=LoadBookmarks()
      if not  equal(x,PROBLEM) then
	DeleteBookmark()
	x=SaveBookmarks()
      end if
    end if

end while

clear_screen()
Pause("Thanks for using S C R O L L   T E X T",175)
clear_screen()
if graphics_mode(-1) then
end if
--===================================================================

